@@ -134,6 +134,7 @@ pub fn functions() -> HashSet<&'static str>
         "all",
         "any",
         "eigenvalues",
+        "eigenvectors",
         "mat",
         "prime",
         "add",
@@ -202,6 +203,13 @@ pub fn functions() -> HashSet<&'static str>
         "doublefactorial",
         "subfactorial",
         "units",
+        "An",
+        "Ap",
+        "eta",
+        "η",
+        "iden",
+        "identity",
+        "quartic",
     ]
     .iter()
     .cloned()
