@@ -79,11 +79,15 @@ pub fn graph(
                     }
                 }
             }
+            if !options.graph
+            {
+                return;
+            }
             options.prec = options.graph_prec;
             func.push(
                 match input_var(
                     &input[i],
-                    vars.clone(),
+                    &vars,
                     &mut Vec::new(),
                     &mut 0,
                     options,
@@ -1043,7 +1047,16 @@ pub fn graph(
         {
             println!("\x1b[G\x1b[K{}ms\x1b[G\n", time.elapsed().as_millis());
         }
-        if fg.show().is_err()
+        if !colors.graphtofile.is_empty()
+        {
+            fg.save_to_png(
+                colors.graphtofile,
+                options.window_size.0 as u32,
+                options.window_size.1 as u32,
+            )
+            .unwrap()
+        }
+        else if fg.show().is_err()
         {
             print!("\x1b[G\x1b[Kno gnuplot\x1b[G\n{}", prompt(options, &colors));
             stdout().flush().unwrap();
@@ -1089,7 +1102,7 @@ pub fn get_list_2d(
     let list = func.0.iter().any(|c| {
         if let Str(s) = c
         {
-            matches!(s.as_str(), "±" | "cubic" | "quadratic")
+            matches!(s.as_str(), "±" | "cubic" | "quadratic" | "quartic")
         }
         else
         {
@@ -1099,7 +1112,7 @@ pub fn get_list_2d(
         c.1.iter().any(|c| {
             if let Str(s) = c
             {
-                matches!(s.as_str(), "±" | "cubic" | "quadratic")
+                matches!(s.as_str(), "±" | "cubic" | "quadratic" | "quartic")
             }
             else
             {
@@ -1388,7 +1401,7 @@ pub fn get_list_3d(
     let list = func.0.iter().any(|c| {
         if let Str(s) = c
         {
-            matches!(s.as_str(), "±" | "cubic" | "quadratic")
+            matches!(s.as_str(), "±" | "cubic" | "quadratic" | "quartic")
         }
         else
         {
@@ -1398,7 +1411,7 @@ pub fn get_list_3d(
         c.1.iter().any(|c| {
             if let Str(s) = c
             {
-                matches!(s.as_str(), "±" | "cubic" | "quadratic")
+                matches!(s.as_str(), "±" | "cubic" | "quadratic" | "quartic")
             }
             else
             {
@@ -1994,7 +2007,7 @@ fn get_data(
             {
                 if re_or_im.1
                 {
-                    points2d[1][0] = points2d[0][0].clone();
+                    points2d[1][0].clone_from(&points2d[0][0].clone());
                     points2d[1].swap(0, 1);
                 }
                 points2d[0].swap(0, 1);
@@ -2003,7 +2016,7 @@ fn get_data(
             {
                 re_or_im.1 = false;
                 points2d[0].swap(0, 1);
-                points2d[0][1] = points2d[1][1].clone();
+                points2d[0][1].clone_from(&points2d[1][1].clone());
                 points2d[1] = Default::default();
             }
             else if func.2.depth
@@ -2045,16 +2058,16 @@ fn get_data(
             if func.2.flat
             {
                 re_or_im.1 = false;
-                points3d[0][1] = points3d[0][2].clone();
-                points3d[0][2] = points3d[1][2].clone();
+                points3d[0][1].clone_from(&points3d[0][2].clone());
+                points3d[0][2].clone_from(&points3d[1][2].clone());
                 points3d[1] = Default::default();
             }
             else if func.2.depth
             {
                 re_or_im.1 = false;
-                points3d[0][0] = points3d[0][1].clone();
-                points3d[0][1] = points3d[0][2].clone();
-                points3d[0][2] = points3d[1][2].clone();
+                points3d[0][0].clone_from(&points3d[0][1].clone());
+                points3d[0][1].clone_from(&points3d[0][2].clone());
+                points3d[0][2].clone_from(&points3d[1][2].clone());
                 points3d[1] = Default::default();
             }
         }
