@@ -42,7 +42,9 @@ FLAGS: --help (this message)\x1b[G\n\
 --depth display 2d complex graphs in 3d with imag #'s going up/down on the z axis\x1b[G\n\
 --surface displays a colored surface(based on z value) for 3d graphing, only supports 1 graph\x1b[G\n\
 --flat display 2d complex graphs like they are on the 2d number line\x1b[G\n\
---scalegraph scales the y part of a 2d graph to the users screen size, setting --windowsize=x,y makes the ratio more accurate\x1b[G\n\n\
+--scalegraph scales the y part of a 2d graph to the users screen size, setting --windowsize=x,y makes the ratio more accurate\x1b[G\n\
+--saveto=[file] saves the graph as a png to the given file, --windowsize=x,y for resolution\x1b[G\n\
+--siunits toggles keeping stuff in si units, a newton will show as 'm s^-2 kg' instead of 'N'\x1b[G\n\n\
 - flags can be executed in runtime just without the dashes\x1b[G\n\
 - \"colors=\" to see color settings\x1b[G\n\
 - \"exit\" to exit the program\x1b[G\n\
@@ -87,14 +89,14 @@ Functions:\x1b[G\n\
 - csc, sec, cot, acsc, asec, acot\x1b[G\n\
 - sinh, cosh, tanh, asinh, acosh, atanh\x1b[G\n\
 - csch, sech, coth, acsch, asech, acoth\x1b[G\n\
-- sqrt, cbrt, square, cube, quadratic(a,b,c) cubic(a,b,c,d) (quadratic and cubic finds the zeros for the given polynomials, you can use cubic(a,b,c,d,1) to only find real solutions for cubics/quadratics)\x1b[G\n\
+- sqrt, cbrt, square, cube, quadratic(a,b,c), cubic(a,b,c,d), quartic(a,b,c,d,e) (finds the zeros for the given polynomial, you can add a '1' to the args to only find real roots)\x1b[G\n\
 - ln, log(base,num), W(k,z) (product log, branch k, defaults to k=0)\x1b[G\n\
 - root(base,exp), sum(var,func,start,end), prod(var,func,start,end)\x1b[G\n\
 - abs, sgn, arg\x1b[G\n\
 - ceil, floor, round, int, frac\x1b[G\n\
 - fact, doublefact, subfact\x1b[G\n\
 - sinc, cis, exp\x1b[G\n\
-- zeta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
+- zeta, eta, gamma, beta, erf, erfc, digamma, ai, multinomial, binomial/bi/C(n,r), P(n,r), pochhammer(x,n)\x1b[G\n\
 - re, im, split(x+yi), next(n,to)\x1b[G\n\
 - factors, prime, isprime, gcd, lcm\x1b[G\n\
 - slog(a,b), ssrt(k,a) (k is lambert w branch)\x1b[G\n\
@@ -106,6 +108,7 @@ Functions:\x1b[G\n\
 - normP(μ,σ,x) (normal distribution pdf) normD(z)/normD(x,μ,σ) (area under curve to the left of z score cdf)\x1b[G\n\
 - betaP(α,β,x) (beta distribution pdf) I(x,a,b) (regularized incomplete beta function, or beta distributions cdf)\x1b[G\n\
 - roll{{a,b,c...}} rolls die, dice{{a,b,c...}} gets the frequency data any amount of different sided die, where a/b/c are number of faces for each die, both also accept {{{{first_dice_face,# of die}},{{second_dice_face,# of die}}...}}\x1b[G\n\
+- An(n,k), Ap(n,t) eulerian numbers and polynomials\x1b[G\n\
 - lim(x,f(x),point (,side)) both sides are checked by default, -1 for left, 1 for right\x1b[G\n\
 - slope(x,f(x),point (,side) (,nth derivitive) (,0) ), can add a 0 to the args to not combine the x and y slopes for parametric equations, same for area\x1b[G\n\
 - area(x,f(x),from,to (,0) ), length(x,f(x),from,to)\x1b[G\n\
@@ -121,7 +124,7 @@ Vector operations/functions:\x1b[G\n\
 - convert to cartesian: car{{vec}} outputs (x, y, z)\x1b[G\n\
 - other functions are applied like sqrt{{2,4}}={{sqrt(2),sqrt(4)}}\x1b[G\n\n\
 Matrix operations/functions:\x1b[G\n\
-- eigenvalues\x1b[G\n\
+- eigenvalues, eigenvectors\x1b[G\n\
 - trace/tr, determinant/det, inverse/inv\x1b[G\n\
 - transpose/trans, adjugate/adj, cofactor/cof, minor\x1b[G\n\
 - part({{mat}},col,row), flatten, sum, prod\x1b[G\n\
@@ -129,7 +132,7 @@ Matrix operations/functions:\x1b[G\n\
 - len, wid\x1b[G\n\
 - max, min, mean, mode\x1b[G\n\
 - iden(n) produces an n dimension identity matrix\x1b[G\n\
-- rotate(theta) produces a rotational matrix\x1b[G\n\
+- rotate(theta), rotate(yaw,pitch,roll) produces a rotational matrix\x1b[G\n\
 - other functions are applied like sqrt{{{{2,4}},{{5,6}}}}={{{{sqrt(2),sqrt(4)}},{{sqrt(5),sqrt(6)}}}}\x1b[G\n\n\
 Constants:\x1b[G\n\
 - c: speed of light, 299792458 m/s\x1b[G\n\
@@ -211,7 +214,7 @@ fn all_units() -> &'static str
 \"ft\" | \"foot\"\x1b[G\n\
 \"in\" | \"inch\"\x1b[G\n\
 \"lb\" | \"pound\"\x1b[G\n\
-\"L\" | \"litre\"\x1b[G\n\
+\"L\" | \"l\" | \"litre\"\x1b[G\n\
 \"Hz\" | \"hertz\"\x1b[G\n\
 \"V\" | \"volt\" | \"voltage\"\x1b[G\n\
 \"°C\" | \"celsius\"\x1b[G\n\
@@ -247,5 +250,15 @@ fn all_units() -> &'static str
 \"sievert\" | \"Sv\"\x1b[G\n\
 \"katal\" | \"kat\"\x1b[G\n\
 \"bit\" | \"b\"\x1b[G\n\
-\"steradian\" | \"sr\""
+\"steradian\" | \"sr\"\x1b[G\n\
+\"atm\"\x1b[G\n\
+\"psi\"\x1b[G\n\
+\"bar\"\x1b[G\n\
+\"tonne\"\x1b[G\n\
+\"hectare\" | \"ha\"\x1b[G\n\
+\"acre\" | \"ac\"\x1b[G\n\
+\"ton\"\x1b[G\n\
+\"oz\"\x1b[G\n\
+\"gallon\" | \"gal\"\x1b[G\n\
+\"floz\""
 }
