@@ -33,26 +33,12 @@ use crossterm::{
 };
 use rug::Complex;
 use std::{
-    env::{args, var},
+    env::args,
     fs::{File, OpenOptions},
     io::{stdin, stdout, BufRead, BufReader, IsTerminal, Write},
     thread::JoinHandle,
     time::Instant,
 };
-//rpn
-//domain coloring
-//matrix exponentiation
-//surface area of a 3d curve
-//maybe nth area, make -nth go into the other function
-//stop vectors or matrixes disipearing graphically by making them have points instead of lines
-//do as much as you can before graphing, make '=' at end show unsimplified, and no = show simplififed
-//allow setting x and y window dimensions of gnuplot, defaulting to what happens now
-//infinitely long greater/lesser than chains
-//have unit powers be fractional, maybe 2 usizes
-//add option to print units in simplified form instead of just in base si units, like N instead of kg m s^-2
-//compress units if makes sense, ie m s^-2 -> m/s^2
-//allow changing the default unit for all, like angles
-//maybe add exchange rates via something like https://www.floatrates.com/daily/usd.json
 #[derive(Clone)]
 pub struct Variable
 {
@@ -73,6 +59,7 @@ pub struct Units
     candela: f64,
     angle: f64,
     byte: f64,
+    usd: f64,
 }
 #[derive(Clone, PartialEq)]
 pub struct Number
@@ -102,6 +89,7 @@ pub struct Colors
     im5col: String,
     im6col: String,
     label: (String, String, String),
+    graphtofile: String,
 }
 impl Default for Colors
 {
@@ -134,6 +122,7 @@ impl Default for Colors
             im5col: "#00aaaa".to_string(),
             im6col: "#aaaa00".to_string(),
             label: ("x".to_string(), "y".to_string(), "z".to_string()),
+            graphtofile: String::new(),
         }
     }
 }
@@ -193,6 +182,7 @@ pub struct Options
     stay_interactive: bool,
     graph_cli: bool,
     units: bool,
+    si_units: bool,
     window_size: (usize, usize),
 }
 impl Default for Options
@@ -238,6 +228,7 @@ impl Default for Options
             stay_interactive: false,
             graph_cli: false,
             units: true,
+            si_units: false,
             window_size: (0, 0),
         }
     }
@@ -248,15 +239,11 @@ fn main()
     let mut options = Options::default();
     let mut args = args().collect::<Vec<String>>();
     let mut default = false;
+    let dir = dirs::config_dir().unwrap().to_str().unwrap().to_owned() + "/kalc";
+    std::fs::create_dir_all(dir.clone()).unwrap();
     {
-        #[cfg(unix)]
-        let file_path = &(var("HOME").unwrap() + "/.config/kalc.config");
-        #[cfg(not(unix))]
-        let file_path = &format!(
-            "C:\\Users\\{}\\AppData\\Roaming\\kalc.config",
-            var("USERNAME").unwrap()
-        );
-        if let Err(s) = file_opts(&mut options, &mut colors, file_path)
+        let file_path = dir.clone() + "/kalc.config";
+        if let Err(s) = file_opts(&mut options, &mut colors, &file_path)
         {
             println!("{}", s);
             std::process::exit(1);
@@ -316,13 +303,7 @@ fn main()
             arg.pop();
         }
     }
-    #[cfg(unix)]
-    let file_path = &(var("HOME").unwrap() + "/.config/kalc.vars");
-    #[cfg(not(unix))]
-    let file_path = &format!(
-        "C:\\Users\\{}\\AppData\\Roaming\\kalc.vars",
-        var("USERNAME").unwrap()
-    );
+    let file_path = dir.clone() + "/kalc.vars";
     let mut vars: Vec<Variable> = if options.allow_vars
         && (options.interactive || options.stay_interactive)
     {
@@ -458,13 +439,7 @@ fn main()
     options.base = base;
     let (mut file, mut unmod_lines) = if options.interactive || options.stay_interactive
     {
-        #[cfg(unix)]
-        let file_path = &(var("HOME").unwrap() + "/.config/kalc.history");
-        #[cfg(not(unix))]
-        let file_path = &format!(
-            "C:\\Users\\{}\\AppData\\Roaming\\kalc.history",
-            var("USERNAME").unwrap()
-        );
+        let file_path = &(dir.clone() + "/kalc.history");
         if File::open(file_path).is_err()
         {
             File::create(file_path).unwrap();
@@ -512,7 +487,7 @@ fn main()
             {
                 (output, funcvar, graphable, varcheck) = match input_var(
                     &input.iter().map(convert).collect::<String>(),
-                    vars.clone(),
+                    &vars,
                     &mut Vec::new(),
                     &mut 0,
                     options,
@@ -876,6 +851,10 @@ fn main()
                     '\x7F' =>
                     {
                         //delete
+                        if placement == input.len()
+                        {
+                            continue;
+                        }
                         if placement - start == 0 && start != 0
                         {
                             start -= 1;
@@ -1625,7 +1604,7 @@ fn main()
             }
             stdout.flush().unwrap()
         }
-        else if options.graph && graphable
+        else if graphable
         {
             let inputs: Vec<String> = input
                 .iter()
