@@ -185,6 +185,7 @@ pub fn set_commands(
                 _ => return Err("bad notation type"),
             }
         }
+        "saveto" => colors.graphtofile = r.to_string(),
         "re1col" => colors.re1col = r.to_string(),
         "im1col" => colors.im1col = r.to_string(),
         "re2col" => colors.re2col = r.to_string(),
@@ -242,11 +243,11 @@ pub fn set_commands(
             }
         }
         "graphcli" | "slowcheck" | "interactive" | "color" | "prompt" | "depth" | "surface"
-        | "flat" | "rt" | "line" | "lines" | "polar" | "frac" | "multi" | "tabbed" | "comma"
-        | "graph" | "units" | "scalegraph" | "debug" | "vars" | "onaxis" | "base" | "ticks"
-        | "decimal" | "deci" | "decimals" | "graphprec" | "graphprecision" | "prec"
-        | "windowsize" | "precision" | "range" | "xr" | "yr" | "zr" | "vrange" | "vxr" | "vyr"
-        | "vzr" | "2d" | "3d" =>
+        | "flat" | "rt" | "siunits" | "line" | "lines" | "polar" | "frac" | "multi" | "tabbed"
+        | "comma" | "graph" | "graphing" | "units" | "scalegraph" | "debug" | "vars" | "onaxis"
+        | "base" | "ticks" | "decimal" | "deci" | "decimals" | "graphprec" | "graphprecision"
+        | "prec" | "windowsize" | "precision" | "range" | "xr" | "yr" | "zr" | "vrange" | "vxr"
+        | "vyr" | "vzr" | "2d" | "3d" =>
         {
             let mut args: Vec<f64> = Vec::new();
             {
@@ -262,7 +263,7 @@ pub fn set_commands(
                         {
                             let parsed = input_var(
                                 &r[last..i],
-                                vars.to_vec(),
+                                vars,
                                 &mut Vec::new(),
                                 &mut 0,
                                 *options,
@@ -286,7 +287,7 @@ pub fn set_commands(
                 }
                 let parsed = input_var(
                     &r[last..],
-                    vars.to_vec(),
+                    vars,
                     &mut Vec::new(),
                     &mut 0,
                     *options,
@@ -313,13 +314,14 @@ pub fn set_commands(
                 "surface" => options.surface = args[0] != 0.0,
                 "flat" => options.flat = args[0] != 0.0,
                 "rt" => options.real_time_output = args[0] != 0.0,
+                "siunits" => options.si_units = args[0] != 0.0,
                 "line" | "lines" => options.lines = args[0] != 0.0,
                 "polar" => options.polar = args[0] != 0.0,
                 "frac" => options.frac = args[0] != 0.0,
                 "multi" => options.multi = args[0] != 0.0,
                 "tabbed" => options.tabbed = args[0] != 0.0,
                 "comma" => options.comma = args[0] != 0.0,
-                "graph" => options.graph = args[0] != 0.0,
+                "graph" | "graphing" => options.graph = args[0] != 0.0,
                 "units" => options.units = args[0] != 0.0,
                 "scalegraph" => options.scale_graph = args[0] != 0.0,
                 "debug" => options.debug = args[0] != 0.0,
@@ -429,7 +431,7 @@ pub fn set_commands(
                                                 let s = split.next().unwrap().to_string();
                                                 let parsed = input_var(
                                                     split.next().unwrap(),
-                                                    vars.to_vec(),
+                                                    vars,
                                                     &mut func_vars,
                                                     &mut 0,
                                                     *options,
@@ -446,7 +448,7 @@ pub fn set_commands(
                                     }
                                     let mut parsed = match input_var(
                                         &unparsed,
-                                        vars.to_vec(),
+                                        vars,
                                         &mut func_vars,
                                         &mut 0,
                                         *options,
@@ -732,13 +734,14 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "surface" => options.surface = !options.surface,
         "flat" => options.flat = !options.flat,
         "rt" => options.real_time_output = !options.real_time_output,
+        "siunits" => options.si_units = !options.si_units,
         "line" | "lines" => options.lines = !options.lines,
         "polar" => options.polar = !options.polar,
         "frac" => options.frac = !options.frac,
         "multi" => options.multi = !options.multi,
         "tabbed" => options.tabbed = !options.tabbed,
         "comma" => options.comma = !options.comma,
-        "graph" => options.graph = !options.graph,
+        "graph" | "graphing" => options.graph = !options.graph,
         "units" => options.graph = !options.units,
         "vars" => options.allow_vars = !options.allow_vars,
         "graphcli" => options.graph_cli = !options.graph_cli,
@@ -818,6 +821,12 @@ pub fn commands(
             stdout.flush().unwrap();
             options.real_time_output = !options.real_time_output;
         }
+        "siunits" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.si_units = !options.si_units;
+        }
         "clear" =>
         {
             execute!(stdout, Clear(ClearType::Purge)).unwrap();
@@ -872,7 +881,7 @@ pub fn commands(
             stdout.flush().unwrap();
             options.comma = !options.comma;
         }
-        "graph" =>
+        "graph" | "graphing" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
@@ -889,7 +898,7 @@ pub fn commands(
             print!("\x1b[G\x1b[A\x1b[K");
             for l in lines
             {
-                print!("{}\x1b[G\n", l);
+                println!("{}\x1b[G", l);
             }
             stdout.flush().unwrap();
         }
@@ -900,8 +909,8 @@ pub fn commands(
             {
                 if !v.unparsed.is_empty()
                 {
-                    print!(
-                        "{}={}\x1b[G\n",
+                    println!(
+                        "{}={}\x1b[G",
                         v.name.iter().collect::<String>(),
                         to_output(
                             &v.unparsed.chars().collect::<Vec<char>>(),
@@ -917,8 +926,8 @@ pub fn commands(
                         Num(n) =>
                         {
                             let n = get_output(*options, colors, n);
-                            print!(
-                                "{}={}{}{}{}\x1b[G\n",
+                            println!(
+                                "{}={}{}{}{}\x1b[G",
                                 v.name.iter().collect::<String>(),
                                 n.0,
                                 n.1,
@@ -941,8 +950,8 @@ pub fn commands(
                                 }
                                 st.push(',');
                             }
-                            print!(
-                                "{}={{{}}}\x1b[G\n",
+                            println!(
+                                "{}={{{}}}\x1b[G",
                                 v.name.iter().collect::<String>(),
                                 st.trim_end_matches(',')
                             )
@@ -969,8 +978,8 @@ pub fn commands(
                                 st.push('}');
                                 st.push(',');
                             }
-                            print!(
-                                "{}={{{}}}\x1b[G\n",
+                            println!(
+                                "{}={{{}}}\x1b[G",
                                 v.name.iter().collect::<String>(),
                                 st.trim_end_matches(',')
                             )
@@ -1007,7 +1016,7 @@ pub fn commands(
                     {
                         if i.contains(r)
                         {
-                            print!("{}\x1b[G\n", i);
+                            println!("{}\x1b[G", i);
                         }
                     }
                     stdout.flush().unwrap();
@@ -1060,6 +1069,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "flat" => format!("{}", options.flat),
         "prompt" => format!("{}", options.prompt),
         "rt" => format!("{}", options.real_time_output),
+        "siunits" => format!("{}", options.si_units),
         "debug" => format!("{}", options.debug),
         "scalegraph" => format!("{}", options.scale_graph),
         "line" => format!("{}", options.lines),
@@ -1068,7 +1078,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "multi" => format!("{}", options.multi),
         "tabbed" => format!("{}", options.tabbed),
         "comma" => format!("{}", options.comma),
-        "graph" => format!("{}", options.graph),
+        "graph" | "graphing" => format!("{}", options.graph),
         "units" => format!("{}", options.units),
         "graphcli" => format!("{}", options.graph_cli),
         "point" => format!("{}", options.point_style),
@@ -1122,6 +1132,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "scic" => colors.sci.to_string(),
         "unitsc" => colors.units.to_string(),
         "bracketc" => bracketcol(&colors.brackets),
+        "saveto" => colors.graphtofile.to_string(),
         "re1col" => formatcol(&colors.re1col),
         "re2col" => formatcol(&colors.re2col),
         "re3col" => formatcol(&colors.re3col),
@@ -1138,7 +1149,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         {
             let input = input_var(
                 &l.replace('_', &format!("({})", last)),
-                vars.to_vec(),
+                vars,
                 &mut Vec::new(),
                 &mut 0,
                 options,
