@@ -8,8 +8,7 @@ use crate::{
     load_vars::set_commands_or_vars,
     math::do_math,
     misc::{
-        clear, clearln, get_terminal_dimensions, handle_err, no_col, parsed_to_string, prompt,
-        to_output,
+        clear, get_terminal_dimensions, handle_err, no_col, parsed_to_string, prompt, to_output,
     },
     options::{equal_to, silent_commands},
     parse::input_var,
@@ -157,7 +156,7 @@ pub fn print_concurrent(
             }
             else
             {
-                clearln(unmodified_input, start, end, options, &colors);
+                clear(unmodified_input, start, end, options, &colors);
                 (0, false, false, false)
             };
         }
@@ -167,7 +166,7 @@ pub fn print_concurrent(
             .iter()
             .collect::<String>()
             .replace('_', &format!("({})", last.iter().collect::<String>())),
-        vars.to_vec(),
+        &vars,
         &mut Vec::new(),
         &mut 0,
         options,
@@ -231,7 +230,6 @@ pub fn print_concurrent(
                 {
                     func.drain(0..=func.iter().position(|c| c == &'(').unwrap());
                     func.pop();
-                    func.pop();
                     for i in func.split(|c| c == &',')
                     {
                         func_vars.push((-1, i.iter().collect()));
@@ -253,7 +251,7 @@ pub fn print_concurrent(
                 }
                 let out = match input_var(
                     &input,
-                    vars.to_vec(),
+                    &vars,
                     &mut func_vars,
                     &mut 0,
                     options,
