@@ -1,5 +1,9 @@
 use crate::{AngleType, Number, Options, Units};
-use rug::{float::Constant::Pi, ops::Pow, Complex};
+use rug::{
+    float::Constant::Pi,
+    ops::{DivRounding, Pow},
+    Complex,
+};
 use std::collections::HashSet;
 impl Units
 {
@@ -27,6 +31,7 @@ impl Units
             candela: self.candela + b.candela,
             angle: self.angle + b.angle,
             byte: self.byte + b.byte,
+            usd: self.usd + b.usd,
         }
     }
     pub fn div(&self, b: &Self) -> Self
@@ -41,6 +46,7 @@ impl Units
             candela: self.candela - b.candela,
             angle: self.angle - b.angle,
             byte: self.byte - b.byte,
+            usd: self.usd - b.usd,
         }
     }
     pub fn pow(&self, b: f64) -> Self
@@ -55,6 +61,7 @@ impl Units
             candela: self.candela * b,
             angle: self.angle * b,
             byte: self.byte * b,
+            usd: self.usd * b,
         }
     }
     pub fn root(&self, b: f64) -> Self
@@ -69,18 +76,294 @@ impl Units
             candela: self.candela / b,
             angle: self.angle / b,
             byte: self.byte / b,
+            usd: self.usd / b,
         }
     }
-    pub fn to_string(&self, options: Options) -> String
+    pub fn to_string(mut self, options: Options) -> String
     {
+        let mut siunits = String::new();
+        if !options.si_units
+        {
+            let farad = self
+                .meter
+                .div_floor(-2.0)
+                .min(self.second.div_floor(4.0))
+                .min(self.kilogram.div_floor(-1.0))
+                .min(self.ampere.div_floor(2.0))
+                .max(0.0);
+
+            if farad != 0.0
+            {
+                self.meter += 2.0 * farad;
+                self.second -= 4.0 * farad;
+                self.kilogram += 1.0 * farad;
+                self.ampere -= 2.0 * farad;
+            }
+            let ohm = self
+                .meter
+                .div_floor(2.0)
+                .min(self.second.div_floor(-3.0))
+                .min(self.kilogram.div_floor(1.0))
+                .min(self.ampere.div_floor(-2.0))
+                .max(0.0);
+            if ohm != 0.0
+            {
+                self.meter -= 2.0 * ohm;
+                self.second += 3.0 * ohm;
+                self.kilogram -= 1.0 * ohm;
+                self.ampere += 2.0 * ohm;
+            }
+            let henry = self
+                .meter
+                .div_floor(2.0)
+                .min(self.second.div_floor(-2.0))
+                .min(self.kilogram.div_floor(1.0))
+                .min(self.ampere.div_floor(-2.0))
+                .max(0.0);
+            if henry != 0.0
+            {
+                self.meter -= 2.0 * henry;
+                self.second += 2.0 * henry;
+                self.kilogram -= 1.0 * henry;
+                self.ampere += 2.0 * henry;
+            }
+            let volt = self
+                .meter
+                .div_floor(2.0)
+                .min(self.second.div_floor(-3.0))
+                .min(self.kilogram.div_floor(1.0))
+                .min(self.ampere.div_floor(-1.0))
+                .max(0.0);
+            if volt != 0.0
+            {
+                self.meter -= 2.0 * volt;
+                self.second += 3.0 * volt;
+                self.kilogram -= 1.0 * volt;
+                self.ampere += 1.0 * volt;
+            }
+            let watt = self
+                .meter
+                .div_floor(2.0)
+                .min(self.second.div_floor(-3.0))
+                .min(self.kilogram.div_floor(1.0))
+                .max(0.0);
+            if watt != 0.0
+            {
+                self.meter -= 2.0 * watt;
+                self.second += 3.0 * watt;
+                self.kilogram -= 1.0 * watt;
+            }
+            let joules = self
+                .meter
+                .div_floor(2.0)
+                .min(self.second.div_floor(-2.0))
+                .min(self.kilogram.div_floor(1.0))
+                .max(0.0);
+            if joules != 0.0
+            {
+                self.meter -= 2.0 * joules;
+                self.second += 2.0 * joules;
+                self.kilogram -= 1.0 * joules;
+            }
+            let newtons = self
+                .meter
+                .div_floor(1.0)
+                .min(self.second.div_floor(-2.0))
+                .min(self.kilogram.div_floor(1.0))
+                .max(0.0);
+            if newtons != 0.0
+            {
+                self.meter -= 1.0 * newtons;
+                self.second += 2.0 * newtons;
+                self.kilogram -= 1.0 * newtons;
+            }
+            let pascal = self
+                .meter
+                .div_floor(-1.0)
+                .min(self.second.div_floor(-2.0))
+                .min(self.kilogram.div_floor(1.0))
+                .max(0.0);
+            if pascal != 0.0
+            {
+                self.meter += 1.0 * pascal;
+                self.second += 2.0 * pascal;
+                self.kilogram -= 1.0 * pascal;
+            }
+            let tesla = self
+                .ampere
+                .div_floor(-1.0)
+                .min(self.second.div_floor(-2.0))
+                .min(self.kilogram.div_floor(1.0))
+                .max(0.0);
+            if tesla != 0.0
+            {
+                self.ampere += 1.0 * tesla;
+                self.second += 2.0 * tesla;
+                self.kilogram -= 1.0 * tesla;
+            }
+            let coulomb = self
+                .ampere
+                .div_floor(1.0)
+                .min(self.second.div_floor(1.0))
+                .max(0.0);
+            if coulomb != 0.0
+            {
+                self.ampere -= 1.0 * coulomb;
+                self.second -= 1.0 * coulomb;
+            }
+            if farad != 0.0
+            {
+                siunits.push_str(
+                    &(" F".to_owned()
+                        + &if farad != 1.0
+                        {
+                            format!("^{}", farad)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if ohm != 0.0
+            {
+                siunits.push_str(
+                    &(" Ω".to_owned()
+                        + &if ohm != 1.0
+                        {
+                            format!("^{}", ohm)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if henry != 0.0
+            {
+                siunits.push_str(
+                    &(" H".to_owned()
+                        + &if henry != 1.0
+                        {
+                            format!("^{}", henry)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if volt != 0.0
+            {
+                siunits.push_str(
+                    &(" V".to_owned()
+                        + &if volt != 1.0
+                        {
+                            format!("^{}", volt)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if watt != 0.0
+            {
+                siunits.push_str(
+                    &(" W".to_owned()
+                        + &if watt != 1.0
+                        {
+                            format!("^{}", watt)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if joules != 0.0
+            {
+                siunits.push_str(
+                    &(" J".to_owned()
+                        + &if joules != 1.0
+                        {
+                            format!("^{}", joules)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if newtons != 0.0
+            {
+                siunits.push_str(
+                    &(" N".to_owned()
+                        + &if newtons != 1.0
+                        {
+                            format!("^{}", newtons)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if pascal != 0.0
+            {
+                siunits.push_str(
+                    &(" Pa".to_owned()
+                        + &if pascal != 1.0
+                        {
+                            format!("^{}", pascal)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if tesla != 0.0
+            {
+                siunits.push_str(
+                    &(" T".to_owned()
+                        + &if tesla != 1.0
+                        {
+                            format!("^{}", tesla)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+            if coulomb != 0.0
+            {
+                siunits.push_str(
+                    &(" C".to_owned()
+                        + &if coulomb != 1.0
+                        {
+                            format!("^{}", coulomb)
+                        }
+                        else
+                        {
+                            String::new()
+                        }),
+                )
+            }
+        }
         format!(
-            "{}{}{}{}{}{}{}{}{}",
+            "{}{}{}{}{}{}{}{}{}{}{}",
+            siunits,
             if self.meter != 0.0
             {
                 " m".to_owned()
                     + &if self.meter != 1.0
                     {
-                        "^".to_owned() + &self.meter.to_string()
+                        format!("^{:.12}", self.meter)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -96,7 +379,10 @@ impl Units
                 " s".to_owned()
                     + &if self.second != 1.0
                     {
-                        "^".to_owned() + &self.second.to_string()
+                        format!("^{:.12}", self.second)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -112,7 +398,10 @@ impl Units
                 " kg".to_owned()
                     + &if self.kilogram != 1.0
                     {
-                        "^".to_owned() + &self.kilogram.to_string()
+                        format!("^{:.12}", self.kilogram)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -128,7 +417,10 @@ impl Units
                 " A".to_owned()
                     + &if self.ampere != 1.0
                     {
-                        "^".to_owned() + &self.ampere.to_string()
+                        format!("^{:.12}", self.ampere)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -144,7 +436,10 @@ impl Units
                 " K".to_owned()
                     + &if self.kelvin != 1.0
                     {
-                        "^".to_owned() + &self.kelvin.to_string()
+                        format!("^{:.12}", self.kelvin)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -160,7 +455,10 @@ impl Units
                 " mol".to_owned()
                     + &if self.mole != 1.0
                     {
-                        "^".to_owned() + &self.mole.to_string()
+                        format!("^{:.12}", self.mole)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -176,7 +474,10 @@ impl Units
                 " cd".to_owned()
                     + &if self.candela != 1.0
                     {
-                        "^".to_owned() + &self.candela.to_string()
+                        format!("^{:.12}", self.candela)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -198,7 +499,10 @@ impl Units
                 .to_owned()
                     + &if self.angle != 1.0
                     {
-                        "^".to_owned() + &self.angle.to_string()
+                        format!("^{:.12}", self.angle)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -214,7 +518,10 @@ impl Units
                 " B".to_owned()
                     + &if self.byte != 1.0
                     {
-                        "^".to_owned() + &self.byte.to_string()
+                        format!("^{:.12}", self.byte)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
                     }
                     else
                     {
@@ -224,7 +531,26 @@ impl Units
             else
             {
                 String::new()
+            },
+            if self.usd != 0.0
+            {
+                " USD".to_owned()
+                    + &if self.usd != 1.0
+                    {
+                        format!("^{:.12}", self.usd)
+                            .trim_end_matches('0')
+                            .trim_end_matches('.')
+                            .to_string()
+                    }
+                    else
+                    {
+                        String::new()
+                    }
             }
+            else
+            {
+                String::new()
+            },
         )
     }
 }
@@ -242,6 +568,7 @@ impl Default for Units
             candela: 0.0,
             angle: 0.0,
             byte: 0.0,
+            usd: 0.0,
         }
     }
 }
@@ -466,6 +793,7 @@ pub fn units() -> HashSet<&'static str>
         "fahrenheit",
         "litre",
         "L",
+        "l",
         "lb",
         "pound",
         "inch",
@@ -504,6 +832,25 @@ pub fn units() -> HashSet<&'static str>
         "ly",
         "nit",
         "nt",
+        "usd",
+        "USD",
+        "$",
+        "¢",
+        "dollar",
+        "cent",
+        "atm",
+        "psi",
+        "bar",
+        "tonne",
+        "hectare",
+        "ha",
+        "acre",
+        "ac",
+        "ton",
+        "oz",
+        "gal",
+        "gallon",
+        "floz",
     ]
     .iter()
     .cloned()
@@ -522,6 +869,12 @@ pub fn to_unit(unit: String, mut num: Complex, options: Options) -> (Number, Opt
         "mol" | "mole" => units.mole = 1.0,
         "cd" | "candela" => units.candela = 1.0,
         "byte" | "B" => units.byte = 1.0,
+        "usd" | "USD" | "$" | "dollar" => units.usd = 1.0,
+        "¢" | "cent" =>
+        {
+            num /= 100;
+            units.usd = 1.0
+        }
         "steradian" | "sr" =>
         {
             match options.angle
@@ -540,7 +893,7 @@ pub fn to_unit(unit: String, mut num: Complex, options: Options) -> (Number, Opt
         }
         "g" | "gram" =>
         {
-            num *= Complex::with_val(options.prec, 10).pow(-3);
+            num /= 1000;
             units.kilogram = 1.0
         }
         "nit" | "nt" =>
@@ -637,9 +990,21 @@ pub fn to_unit(unit: String, mut num: Complex, options: Options) -> (Number, Opt
             num *= 45359237;
             num /= 100000000;
         }
-        "L" | "litre" =>
+        "L" | "l" | "litre" =>
         {
-            num *= Complex::with_val(options.prec, 10).pow(-3);
+            num /= 1000;
+            units.meter = 3.0;
+        }
+        "floz" =>
+        {
+            num *= 1420653;
+            num /= 50000000000u64;
+            units.meter = 3.0;
+        }
+        "gallon" | "gal" =>
+        {
+            num *= 473176473;
+            num /= 125000000000u64;
             units.meter = 3.0;
         }
         "Hz" | "hertz" => units.second = -1.0,
@@ -735,6 +1100,56 @@ pub fn to_unit(unit: String, mut num: Complex, options: Options) -> (Number, Opt
             units.meter = -1.0;
             units.second = -2.0;
         }
+        "atm" =>
+        {
+            num *= 101325;
+            units.kilogram = 1.0;
+            units.meter = -1.0;
+            units.second = -2.0;
+        }
+        "psi" =>
+        {
+            num *= 6894757;
+            num /= 1000000;
+            units.kilogram = 1.0;
+            units.meter = -1.0;
+            units.second = -2.0;
+        }
+        "bar" =>
+        {
+            num *= 100000;
+            units.kilogram = 1.0;
+            units.meter = -1.0;
+            units.second = -2.0;
+        }
+        "tonne" =>
+        {
+            num *= 1000;
+            units.kilogram = 1.0;
+        }
+        "hectare" | "ha" =>
+        {
+            num *= 10000;
+            units.meter = 2.0;
+        }
+        "acre" | "ac" =>
+        {
+            num *= 316160658;
+            num /= 78125;
+            units.meter = 2.0;
+        }
+        "ton" =>
+        {
+            num *= 45359237;
+            num /= 50000;
+            units.kilogram = 1.0;
+        }
+        "oz" =>
+        {
+            num *= 45359237;
+            num /= 1600000000;
+            units.kilogram = 1.0;
+        }
         "Ω" | "ohm" =>
         {
             units.kilogram = 1.0;
